<?php
require_once 'DataTransfer.php';
require_once 'stringVariance.php';

/**
 * Submit user feedback to secure Netblog Server.
 */
class nbFeedbackSubmit {
	
	/**
	 * Set feedback message text
	 *
	 * @param string $msg
	 */
	function setMessage( $msg ) {
		$this->data['message'] = trim(strip_tags($msg));		
	}
	
	/**
	 * Set feedback type
	 *
	 * @param string $type Any of 'happy' | 'sad'
	 */
	function setFeedbackType( $type ) {
		if($type!='happy' && $type!='sad') $type = 'happy';
		$this->data['type'] = $type;
	}
	
	/**
	 * Send feedback to official Netblog feedback server.
	 *
	 * @return bool
	 */
	function send() {
		if(Netblog::options()->getPrivacyLevel()=='ultra') {
			$this->errno = 501;
			$this->error = 'Strict Privacy Level';
			return false;
		}
		if($this->data['message']=='') {
			$this->errno = 502;
			$this->error = 'Missing Feedback Message';
			return false;
		}
		if(strlen($this->data['message']) < 10) {
			$this->errno = 503;
			$this->error = 'Feedback Message to short; at least 10 characters are required!';
			return false;
		}
		if(strlen($this->data['message']) > 255 ) {
			$this->errno = 503;
			$this->error = 'Feedback Message exceeds maximum size of 255 characters!';
			return false;
		}
		$svar = new stringVariance($this->data['message']);
		if($svar->CountDiffChars() < 15) {
			$this->errno = 504;
			$this->error = 'Feedback Message has been categorized as spam. Please write more details.';
			return false;
		}
		
		$this->appendAppData();
		$engine_uri = $this->host . "submit.php";
		Netblog::log("Sending user feedback...");
		Netblog::log("Transfering ".round(strlen(http_build_query($this->data))/1024,3)."KB encrypted data to $engine_uri");
		$result = DataTransfer::SubmitPost($engine_uri, $this->data);
					 
		// CONTACT SERVER AND RETRIEVE RESPONSE
		if($result['content']) {
			$this->parse($result['content']);			
			if($this->errno==0) {
				Netblog::logSuccess("User feedback has been sended");
				return true;
			}
		} else if($result['error']) {
			$this->errno = $result['errno'];
			$this->error = $result['error'];
		} else {
			$this->errno = 503;
			$this->error = 'Service Unavailable';
		}
		Netblog::logError("Failed to send user feedback (Error ".$this->errno.': '.$this->error.')');
		return false;
	}
	
	/**
	 * Append application and host data for better support
	 *
	 */
	private function appendAppData() {
		$this->data['netblog_client'] = Netblog::getClientVersion();			// for statistics and to improve support
		$this->data['netblog_server'] = Netblog::getServerVersion();
		$this->data['host'] = 'wordpress';
		$this->data['host_version'] = get_bloginfo('version');					// helps to find bugs and to improve wp support
		$this->data['host_language'] = get_bloginfo('language');				// to improve multi-language support
		
		if(Netblog::options()->getPrivacyLevel()=='none') {
			$this->data['host_url'] = get_bloginfo('url');							// optional: those who submit a lot of useful feedback
			$this->data['host_description'] = get_bloginfo('description');				// may be promoted to beta tester
			$this->data['host_name'] = get_bloginfo('name');							// - these data are not public (feedback site)
																						// - send an email (feedback site) to prevent storage
		}																				//   of these data
		
		if(Netblog::options()->getPrivacyLevel()=='medium' || Netblog::options()->getPrivacyLevel()=='none') {
			$this->data['host_charset'] = get_bloginfo('charset');
			
			$ct = current_theme_info();
			$this->data['host_theme_name'] = $ct->name;
			$this->data['host_theme_title'] = $ct->title;
			$this->data['host_theme_version'] = $ct->version;
			$this->data['host_theme_parent'] = $ct->parent_theme;
			$this->data['host_theme_template'] = $ct->template;
			
			$pls = get_plugins();$i=0;
			foreach($pls as $pl) {
				$this->data['host_plugin_'.$i.'_name'] = $pl['Name'];
				$this->data['host_plugin_'.$i.'_version'] = $pl['Version'];
				$this->data['host_plugin_'.$i.'_network'] = $pl['Network'];
				$this->data['host_plugin_'.$i.'_uri'] = $pl['PluginURI'];
				$i++;
			}		
			$this->data['host_multisite'] = is_multisite();		
			
			$this->data['useragent'] = $_SERVER['HTTP_USER_AGENT'];					// for better testing of new versions		
			$this->data['php_version'] = phpversion();
			$this->data['mysql_version'] = nbdb::getServerVersion();
			$this->data['php_extensions'] = implode('|',get_loaded_extensions());
		}
	}
	
	/**
	 * Parse special-formatted string from server response.
	 *
	 * @param string $response
	 * @return void
	 */
	private function parse( $response )
	{
		$this->error = '';
		$this->errno = 0;
		$this->resp = $response;
							
		if( $response == '1' || $response == 'ok' ) return;
		if( trim(strlen($response))==0 ) {
			$this->errno = 502;
			$this->error = __('Invalid Server Response','netblog');	
		}
		$string = base64_decode($response);
		
		// TEST FOR ERRORS
		if( ($perr=stripos($string,'error')) !== false ) {
			$perr += 5;
			$this->errno = trim(substr($string,$perr, ($p2=stripos($string,':',$perr))-$perr) );
			$this->error = trim(substr($string,$p2+1));
			return;
		}
	}
	
	/**
	 * Get error number
	 *
	 * @return string
	 */
	function getErrno() { return $this->errno; }
	
	/**
	 * Get error message
	 *
	 * @return string
	 */
	function getErrorMessage() { return $this->error; }
	
	private $data = array();
	private $error = '';
	private $errno = 0;
	private $host = 'http://netblog2.benjaminsommer.com/feedbacks/'; 
	//private $host = 'http://localhost/nbfeedback/';
	
	public $resp = '';
}


function netblog_feedback_smilies() 
{
	?>
	<div class="feedback-frown" onclick="nbpop_load('nbpopup_feedback_frown')"></div>
	<div class="feedback-smile" onclick="nbpop_load('nbpopup_feedback_smile')"></div>
		
	<div class="netblog-container" id="nbpopup_feedback_smile" style="display:none">
	<div class="nbarea-popup-bg opac50"></div>
	<div class="nbarea-popup-bg2 opac65"></div>
		<div style="width:650px; height:auto;" id="nbpopup_feedback_smile_wnd" class="nbpopup noopac shadow4">
			<form method="post" id="nbarea-feedback-smile-form" />
			
			<h2>Netblog Made Me Happy Because...</h2>			
			<input type="hidden" name="netblog_feedback_type" value="smile" />			
			<div class="feedback-smile-large fltlft"></div>
			<div style="margin-left: 170px;">	
				Super! It's great to hear that you like Netblog <?php echo Netblog::getClientVersion() ?>. <br />
				Please explain what you specifically like!		
				<textarea style="width:475px;height:70px" name="netblog_feedback_msg" onkeyup="input_strip(this,200)"></textarea>			
				<div class="note-bg-round">Please ensure that there is no personally identifiable information here.
					<?php echo '<br /><b>Privacy Setting</b>: '.ucfirst(Netblog::options()->getPrivacyLevel());
						echo '<br /><b>Privacy Details</b>:<a>List of submitted data</a>'; ?></div>			
			</div>
			<div class="nofloat noflt"></div>			
			
			<div style="text-align:center; margin: 20px;">
				<input type="button" value="Submit Feedback" style="margin: 0 10px; cursor:pointer;"
						onclick="sendFeedback('nbarea-feedback-smile-form')" />
				<input type="button" value="Cancel" style="margin: 0 10px; cursor: pointer" onclick="nbpop_unload()" />  
			</div>
			
			</form>				
		</div>		 
	</div>

	<div class="netblog-container" id="nbpopup_feedback_frown" style="display:none">
	<div class="nbarea-popup-bg opac50"></div>
	<div class="nbarea-popup-bg2 opac65"></div>
		<div style="width:650px; height:auto;" id="nbpopup_feedback_frown_wnd" class="nbpopup noopac shadow4">
			<form method="post" id="nbarea-feedback-frown-form" />
						
			<h2>Netblog Made Me Sad Because...</h2>			
			<input type="hidden" name="netblog_feedback_type" value="frown" />			
			<div class="feedback-frown-large fltlft"></div>
			<div style="margin-left: 170px;">	
				We are sorry that you have not had a good experience with our plugin. Please tell us why, so we can fix it! 		
				<textarea style="width:475px;height:70px" name="netblog_feedback_msg" onkeyup="input_strip(this,200)"></textarea>		
				<div class="note-bg-round">Please ensure that there is no personally identifiable information here.
					<?php echo '<br /><b>Privacy Setting</b>: '.ucfirst(Netblog::options()->getPrivacyLevel());
						echo '<br /><b>Privacy Details</b>:<a>List of submitted data</a>'; ?></div>			
			</div>
			<div class="nofloat noflt"></div>			
			
			<div style="text-align:center; margin: 20px;">
				<input type="button" value="Submit Feedback" style="margin: 0 10px; cursor:pointer;"
						onclick="sendFeedback('nbarea-feedback-frown-form')" />
				<input type="button" value="Cancel" style="margin: 0 10px; cursor: pointer" onclick="nbpop_unload()" />  
			</div>
			
			</form>				
		</div>		 
	</div>
	
	<script type="text/javascript">
	<!--
		function sendFeedback( formId ) {		
			var infos = jQuery("#"+formId).serialize();
			
			jQuery(document).ready(function($) {
				var data = {
					action: 'nbfeedback_send',
					form_encoded: infos
				};
				jQuery.post(ajaxurl, data, function(r) {
					r = r.trim();
					if(r=='ok') {
						nbpop_unload();
					} else {
						alert(r);
					}
				});
			});
			
		}
	//-->
	</script>
	
	<?php
}

add_action('wp_ajax_nbfeedback_send','nbfeedback_send');
function nbfeedback_send() {
	$formStr = $_POST['form_encoded'];	
	parse_str(html_entity_decode($formStr),$form);
	
	$feed = new nbFeedbackSubmit();
	$feed->setMessage($form['netblog_feedback_msg']);
	$feed->setFeedbackType($form['netblog_feedback_type']=='smile' ? 'happy' : 'sad');
	if( $feed->send() )
		echo 'ok';
	else echo 'Error '.$feed->getErrno().': '.$feed->getErrorMessage();
	die();
}
?>